@echo off

::
:: This script installs uberAgent manually
::
:: Alternatively you can install the MSI through other means
:: 
:: Requirements:
::
:: - The following files must be present in the same directory as this batch file
::    - uberAgent-32.msi
::    - uberAgent-64.msi
::
:: Return values:
::
::    1  ->    Required file is missing
::    x  ->    Return value from msiexec
::

:: Check prerequisites
if not exist "%~dp0uberAgent-32.msi" (
   echo ERROR: 32-bit MSI not found 1>&2
	exit /b 1
)
if not exist "%~dp0uberAgent-64.msi" (
   echo ERROR: 64-bit MSI not found 1>&2
	exit /b 1
)

::
:: Select 32- or 64-bit version according to the bitness of the OS
::
if defined programfiles(x86) (
   set installer=%~dp0uberAgent-64.msi
) else (
   set installer=%~dp0uberAgent-32.msi
)

::
:: Copy license file and custom configuration files
::
if not exist "%ProgramData%\vast limits" (
   mkdir "%ProgramData%\vast limits"
   icacls "%ProgramData%\vast limits" /inheritance:r /grant:r "*S-1-5-32-544:(OI)(CI)F" /grant:r "*S-1-5-18:(OI)(CI)F"
)   

if exist "%~dp0uberAgent*.lic" (
   xcopy "%~dp0uberAgent*.lic" "%ProgramData%\vast limits\uberAgent\License cache\" /y /s /v /e
)

if exist "%~dp0uberAgent*.conf" (
   xcopy "%~dp0uberAgent*.conf" "%ProgramData%\vast limits\uberAgent\Configuration\" /y /s /v /e
)

::
:: Run the installer
::
echo Running the installer... 1>&2

start /wait msiexec /i "%installer%" /l*v "%temp%\uberAgentMsi.log" 1>nul

echo ...done! 1>&2

exit /b %errorlevel%