#!/bin/bash

# This script installs uberAgent silently
#
# It is a method of installing uberAgent through Splunk's Deployment server
# Alternatively you can simply install the PKG file through other means
# 
# The following files must/can be present in the same directory as this shell script
#    - uberAgent.pkg (mandatory)
#    - uberAgent*.lic (optional)
#    - uberAgent.conf (optional)

# Define variables
RELATIVE_PATH=$(dirname "$0")
ABSOLUTE_PATH=$(cd "$RELATIVE_PATH" && pwd)
APPLICATION_SUPPORT_DIR="/Library/Application Support/uberAgent"

# Ask for superuser rights
if [ $EUID != 0 ]; then
   echo "Please run this script with root privileges. Exiting script" >&2
   exit 1
fi

# Check prerequisites
echo "Checking presence of uberAgent.pkg in '$ABSOLUTE_PATH'" >&2
if ! [ -f "$ABSOLUTE_PATH/uberAgent.pkg" ]; then
   echo "uberAgent.pkg not found. Exiting script" >&2
   exit 1
fi

# Application Support directory
echo "Preparing application support directory" >&2
if [ ! -d "$APPLICATION_SUPPORT_DIR" ]
then
  echo "Found no previous uberAgent installation" >&2
  echo "Setting up application support directory at $APPLICATION_SUPPORT_DIR" >&2
  mkdir -p "$APPLICATION_SUPPORT_DIR"
  chmod 700 "$APPLICATION_SUPPORT_DIR"
  chown root:wheel "$APPLICATION_SUPPORT_DIR"
else
  echo "Found previous uberAgent installation. No folder creation neccesary" >&2
fi

# Configuration file 
echo "Looking for custom uberAgent configuration in '$ABSOLUTE_PATH'" >&2
if [ -f "$ABSOLUTE_PATH/uberAgent.conf" ]; then
   echo "uberAgent.conf found. Copying" >&2
   cp "$ABSOLUTE_PATH/uberAgent.conf" "$APPLICATION_SUPPORT_DIR/uberAgent.conf"
else 
   echo "uberAgent.conf not found" >&2
fi

# License file
echo "Looking for uberAgent license file(s) in '$ABSOLUTE_PATH'" >&2
if [ -n "$(find "$ABSOLUTE_PATH" -name '*.lic' | head -1)" ]; then
   echo "License file(s) found. Copying" >&2
   cd "$ABSOLUTE_PATH" && cp ./*.lic "$APPLICATION_SUPPORT_DIR/"
else
   echo "No license files found" >&2
fi

# Run the installer
echo "uberAgent for macOS silent installation started" >&2
echo "Full path to pkg is '$ABSOLUTE_PATH/uberAgent.pkg'" >&2
installer -pkg "$ABSOLUTE_PATH/uberAgent.pkg" -target / >&2

exit 0