//
// Table cell renderer
//
require([
   'jquery',
   'splunkjs/mvc/tableview',
   'splunkjs/mvc',
   'underscore',
   'splunkjs/mvc/simplexml/ready!'
],function(
   $,
   TableView,
   mvc,
   _
)
{
   // Functions
   function UnsetToken(name) {
      var defaultTokenModel = mvc.Components.getInstance('default', {create: true});
      var submittedTokenModel = mvc.Components.getInstance('submitted', {create: true});

      defaultTokenModel.unset(name);
      submittedTokenModel.unset(name);
   }

   // Page description
   $('.dashboard-header > p.description').text (gettext("This dashboard lists processes that were tagged for risky behavior. The data is enriched with MITRE ATT&CK® information."));


   // Format commandline column
   var TimebarCellRenderer = TableView.BaseCellRenderer.extend(
   {
      // Called to check whether to invoke our custom renderer
      canRender: function(cell)
      {
         return (cell.field === 'Command line' || cell.field === 'Path' );
      },
      
      // The render function is only called when canRender returns true
      render: function($td, cell)
      {
         if (cell.value === null)
         {
            return;
         }

         if (cell.field === 'Command line')
         {
            var message = cell.value;
            var tooltip = cell.value;
            
            message = message.replace ("\\SystemRoot\\System32\\", "...\\");
            message = message.replace ("%SystemRoot%\\system32\\", "...\\");
            message = message.replace ("\\??\\C:\\Windows\\system32\\", "...\\");
            message = message.replace ("C:\\Windows\\system32\\", "...\\");
            message = message.replace ("C:\\Program Files (x86)\\", "...\\");
            message = message.replace ("C:\\Program Files\\", "...\\");
            message = message.replace ("C:\\Windows\\SysWOW64\\", "...\\");
            message = message.replace ("C:\\Windows\\", "...\\");
            if (message.length > 50)
            {
               message = message.substring(0, 49) + "...";
            }
               
            $td.html(_.template('<div title="<%- tooltip %>"> <%- text %></div>',
            {
               text:       message,
               tooltip:    tooltip
            }));
         }

      }
   });

   mvc.Components.get('Table_Panel31').getVisualization(function(tableView)
   {
      // Register custom cell renderer
      tableView.table.addCellRenderer(new TimebarCellRenderer());
      
      // Force the table to re-render
      tableView.table.render();
   });

   // When the DOM is ready, move the modal buttons where we want it to be displayed
   $("#Panel21_Input2").find("[id*='label']").after($("#btnModalInfo1"));
   $("#Panel31").find("h2").after($("#btnModalInfo2"));

   // Custom drilldowns
   var TablePanel31 = mvc.Components.getInstance ("Table_Panel31");
   TablePanel31.on ("click:row", drilldown31);

   function drilldown31 (event)
   {
      // Don't do any further event processing
      event.preventDefault ();
      
      if (event.data["click.name2"] == "ATT&CK techniques")
      {
         var DrilldownFilterBy31 = event.data["click.value2"];         
         if (DrilldownFilterBy31 != null)
         {
            SetToken (mvc, "ShowPanel32", true, submit);
            SetToken (mvc, "DrilldownFilterBy31", DrilldownFilterBy31, submit);
         }
      }
      else
      {
         var earliest = GetToken (mvc, "earliest");
         var latest = GetToken (mvc, "latest");
         
         var drilldownUrl = "process_tree";
         drilldownUrl += "?earliest=" + encodeURIComponent (earliest);
         drilldownUrl += "&latest=" + encodeURIComponent (latest);
         drilldownUrl += "&form.HostFilter=" + encodeURIComponent ((event.data["row.Host"]));
         drilldownUrl += "&form.Panel41ProcessNameFilter=" + encodeURIComponent ((event.data["row.ProcName"]));

         window.location = drilldownUrl;
      }

   }

   // Attach click event handlers to dashboard controls
   $(document).on("click", "#Panel32CloseBtn", function() {
      UnsetToken("ShowPanel32");
      UnsetToken("DrilldownFilterBy31");
      UnsetToken("MitreName");
      UnsetToken("MitreID");
      UnsetToken("MitreURL");
      UnsetToken("MitreDesc");
      UnsetToken("MitreDet");
  });


  var submittedTokens = mvc.Components.get('submitted');
   // Listen for a change to the token MitreDesc value
   submittedTokens.on("change:MitreDesc", function(model, MitreDesc, options) {
      var MitreDescJS=submittedTokens.get("MitreDesc");
      if (MitreDescJS!==undefined)
      {
         $("#Panel32MitreDesc").html(MitreDescJS);
      }
   });

   // Listen for a change to the token MitreDet value
   submittedTokens.on("change:MitreDet", function(model, MitreDet, options) {
      var MitreDetJS=submittedTokens.get("MitreDet");
      if (MitreDetJS!==undefined)
      {
         $("#Panel32MitreDet").html(MitreDetJS);
      }
   });

});